@@ -1,7 +1,7 @@
 /*
  * server.c -- nsd(8) network input/output
  *
- * Copyright (c) 2001-2004, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
@@ -193,9 +193,10 @@ restart_child_servers(struct nsd *nsd)
 			nsd->children[i].pid = fork();
 			switch (nsd->children[i].pid) {
 			case 0: /* CHILD */
+				nsd->server_kind = nsd->children[i].kind;
+				nsd->mode = NSD_RUN;
 				nsd->pid = 0;
 				nsd->child_count = 0;
-				nsd->server_kind = nsd->children[i].kind;
 				server_child(nsd);
 				/* NOTREACH */
 				exit(0);
@@ -911,15 +912,11 @@ handle_tcp_reading(netio_type *netio,
 	assert(buffer_position(data->query->packet) == data->query->tcplen);
 
 	/* Account... */
-#ifndef INET6
-	STATUP(data->nsd, ctcp);
-#else
 	if (data->query->addr.ss_family == AF_INET) {
 		STATUP(data->nsd, ctcp);
 	} else if (data->query->addr.ss_family == AF_INET6) {
 		STATUP(data->nsd, ctcp6);
 	}
-#endif
 
 	/* We have a complete query, process it.  */
 
